import static org.junit.Assert.*;
import org.junit.Test;

import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

public class JUnitTest {

	final String newLine = System.getProperty("line.separator");
	String pathPrefix = null;
	
	public JUnitTest () {
		pathPrefix = new File("").getAbsolutePath() + "/test/files/";
	}
	
	private static String normalizeLineEnds(String s) {
	    return s.replace("\r\n", "\n").replace('\r', '\n');
	}
	
	private void checkEquality(String testFileName) {		
		try {
			//	
			// Define the input and the output files.
			//
			File inputFile  = new File (pathPrefix + File.separator + testFileName + ".in.txt");
			File outputFile = new File (pathPrefix + File.separator + testFileName + ".out.txt");
			//
			// Read the expected output
			//
			String expectedResult = "";
			BufferedReader buffReader = new BufferedReader(new FileReader(outputFile.getPath()));
	        String line = buffReader.readLine();
	        while (line != null) {
	        	expectedResult += line + '\n';
	            line = buffReader.readLine();
	        }
	        buffReader.close();
	        
			//
			// Define the input and output streams
			// 
			final ByteArrayOutputStream baos = new ByteArrayOutputStream();
			FileInputStream in = new FileInputStream(inputFile);		
			PrintStream out = new PrintStream(baos);			
			//
			// Perform the test
			//		
			ReadAndWrite rw = new ReadAndWrite();
			rw.readAndSolve(in, out);
			//
			// Flush the output and obtain the data
			//
			out.flush();
			out.close();
			String result = baos.toString();
			//
			// Now check for the actual assertion
			//
			assertEquals(normalizeLineEnds(expectedResult), normalizeLineEnds(result));		
		} catch (IOException e) {			
			fail(e.getMessage());
			
		}		
	}
	
	@Test
	public void testTinyAntisymmetric() {
		checkEquality("antisymetric_tiny"); // is antisymmetric
		checkEquality("dag_a_tiny"); // likely is not antisymmetric
	}
	
	@Test
	public void testLargeAntisymmetric() {
		// reference solution runs in 0.69 s on i7-8650U
		// and 1 s on Judge
		checkEquality("antisymetric_large"); // is antisymmetric
		checkEquality("complete_a"); // not antisymmetric
		checkEquality("dag_a_large"); // likely is not antisymmetric
	}
	
	@Test
	public void testTinyTransitive() {
		checkEquality("transitive_tiny");
	}
	
	@Test
	public void testTransitive() {
		checkEquality("transitive");
	}
	
	@Test
	public void testTinyCycle() {
		checkEquality("antisymetric_c_tiny"); // likely contains cycle
		checkEquality("dag_c_tiny"); // without cycles
	}
	
	@Test
	public void testLargeCycle() {
		// reference solution runs in 0.75 s on i7-8650U
		// and 1 s on Judge
		checkEquality("antisymetric_c_large"); // likely contains cycle
		checkEquality("complete_c"); // with cycles
		checkEquality("dag_c_large"); // without cycles
	}
	
	@Test
	public void testSingle() {
		/**
		 * Note that this test will always pass.
		 * Use this test to debug a single wrong graph.
		 * 
		 * Copy the line representing some graph from the input to string
		 * <code>stringGraphToShow</code>. The test will run selected method 
		 * <code>// select your operation here</code>, and it will generate GraphViz format
		 * representation of the input. You can find this GraphViz file in the directory
		 * with test files, as graph.dot.
		 * 
		 * To visualize the file, run in terminal graphviz fileName.dot (see tutorial pdf)
		 */
		String stringGraphToShow = "l\n" + 
				"5\n" + 
				"4\n" + 
				"1 0\n" + 
				"2 0\n" + 
				"3 0\n" + 
				"4 0\n";
		char testMode = 'c'; // select mode: 'a' (antisymmetric), 'c' (cycle), 't' (transitive), or 's' (star)
		
		FastReader s = new FastReader(new ByteArrayInputStream(stringGraphToShow.getBytes()));
		
		if (testMode == 's') {
			char inMode = s.nextChar();
			if (inMode != 'l') {
				System.err.println("Wrong mode!");
			}
			int size = s.nextInt();
			int nEdges = s.nextInt();
			Edge[] edges = new Edge[nEdges];
			for (int j = 0; j < nEdges; j++) {
				edges[j] = new Edge(s.nextInt(), s.nextInt());
			}

			StarDiGraph g = new StarDiGraph(size, edges);
			boolean b = g.containsStar(); // observe result from containsStar
			
			String filename = pathPrefix + "/graph.dot"; 
			
			try {
		    	PrintWriter writer = new PrintWriter(filename, "UTF-8");
				
		    	writer.write("digraph G {\n");
		    	
		    	
				for (Edge e: g.edges) {
					writer.write("\"" + e.u + "\" -> \"" + e.v + "\"\n");
				}
				writer.write("}\n");
				
		        writer.close();
		    } catch (FileNotFoundException e) {
				// Auto-generated catch block
				e.printStackTrace();
			} catch (UnsupportedEncodingException e) {
				// Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			char inMode = s.nextChar();
			int vertices = s.nextInt();
			boolean[][] matrix = new boolean[vertices][vertices];
			
			switch (inMode) {
			case 'm':
				for (int u = 0; u < vertices; u++) {
					for (int v = 0; v < vertices; v++) {
						matrix[u][v] = s.nextInt() == 1;
					}
				}
				break;
			case 'l':
				int edges = s.nextInt();
				for (int j = 0; j < edges; j++) {
					matrix[s.nextInt()][s.nextInt()] = true;
				}
				break;
			case 'c':
				for (int u = 0; u < vertices; u++) {
					for (int v = 0; v < vertices; v++) {
						matrix[u][v] = true;
					}
				}
				break;

			default:
				break;
			}
			
			
			DiGraph g = new DiGraph(matrix);
			
			switch (testMode) {
			case 'a':
				boolean b = g.isAntisymmetricRelation(); // observe result from isAntisymmetricRelation
				break;
			case 't':
				b = g.isTransitiveRelation(); // observe result from isTransitiveRelation
				break;
			case 'c':
				b = g.containsCycle(); // observe result from containsCycle
				break;

			default:
				break;
			}
			
			String filename = pathPrefix + "/graph.dot"; 
			
			try {
		    	PrintWriter writer = new PrintWriter(filename, "UTF-8");
				
		    	writer.write("digraph G {\n");
		    	
		    	writer.write("{\n node [style=filled]\n");
				for (int i = 0; i < g.diGraph.length; ++i) {
					writer.write("\"" + i + "\" [fillcolor=" + g.getNodeColor(i) + "]\n");
				}
		    	writer.write("}\n");
		    	
		    	
				for (int i = 0; i < g.diGraph.length; ++i) {
					for (int j = 0; j < g.diGraph.length; ++j) {
						if (g.diGraph[i][j]) {
							writer.write("\"" + i + "\" -> \"" + j + "\"\n");
						}
					}
				}
				writer.write("}\n");
				
		        writer.close();
		    } catch (FileNotFoundException e) {
				// Auto-generated catch block
				e.printStackTrace();
			} catch (UnsupportedEncodingException e) {
				// Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
}







